import psycopg2
from psycopg2 import sql
from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
from typing import Dict, List, Union, Any, Optional

class ConnectionManager:
    """Context manager for handling database connections"""
    def __init__(self, conn_params: Dict[str, Any]):
        self.conn_params = conn_params
        self.connection = None
        
    def __enter__(self):
        self.connection = psycopg2.connect(**self.conn_params)
        return self.connection
        
    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.connection:
            self.connection.close()

def connect(host: str, user: str, passwd: str, port: int = 5432, db: str = None) -> ConnectionManager:
    """Connect to PostgreSQL server with automatic connection management"""
    conn_params = {
        'host': host,
        'user': user,
        'password': passwd,
        'port': port,
        'dbname': db if db else user  # Default to username if db not specified
    }
    return ConnectionManager(conn_params)

def create_db(conn: ConnectionManager, db_name: str) -> None:
    """
    Create a new PostgreSQL database
    
    :param conn: Active connection manager (must be connected to template1 or postgres db)
    :param db_name: Name of the database to create
    """
    conn.connection.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
    query = sql.SQL("CREATE DATABASE {}").format(sql.Identifier(db_name))
    
    with conn as connection:
        execute_sql(connection, query)

def execute_sql(conn: psycopg2.extensions.connection, query: Union[str, sql.Composed], 
                params: Optional[tuple] = None, fetch: bool = False) -> Optional[List[Dict]]:
    """Safe SQL execution handler with error catching"""
    try:
        with conn.cursor() as cursor:
            cursor.execute(query, params)
            if fetch:
                rows = cursor.fetchall()
                colnames = [desc[0] for desc in cursor.description]
                return [dict(zip(colnames, row)) for row in rows]
            conn.commit()
    except Exception as e:
        conn.rollback()
        raise RuntimeError(f"Database operation failed: {str(e)}") from e

def create_table(conn: ConnectionManager, table_name: str, columns: Dict[str, str]) -> None:
    """
    Create table with schema validation and IF NOT EXISTS protection
    
    :param conn: Active connection manager
    :param table_name: Name of table to create (can be comma-separated for multiple tables)
    :param columns: Dict of {column_name: data_type}
    """
    if not isinstance(columns, dict) or not columns:
        raise ValueError("Columns must be a non-empty dictionary")
        
    def _create_single_table(t_name, cols):
        columns_sql = sql.SQL(', ').join(
            sql.SQL("{} {}").format(
                sql.Identifier(col_name),
                sql.SQL(data_type)
            ) for col_name, data_type in cols.items()
        )
        
        query = sql.SQL("CREATE TABLE IF NOT EXISTS {} ({})").format(
            sql.Identifier(t_name),
            columns_sql
        )
        execute_sql(conn.connection, query)

    tables = [t.strip() for t in table_name.split(',')]
    with conn as connection:
        for table in tables:
            _create_single_table(table, columns)

def insert_data(conn: ConnectionManager, table_name: str, data: Dict[str, Any]) -> None:
    """
    Insert data with parameterized queries to prevent SQL injection
    
    :param conn: Active connection manager
    :param table_name: Target table name
    :param data: Dict of {column: value}
    """
    if not data:
        raise ValueError("Data cannot be empty")
        
    columns = data.keys()
    values = [data[col] for col in columns]
    
    query = sql.SQL("INSERT INTO {} ({}) VALUES ({})").format(
        sql.Identifier(table_name),
        sql.SQL(', ').join(map(sql.Identifier, columns)),
        sql.SQL(', ').join([sql.Placeholder()] * len(columns))
    )

    with conn as connection:
        execute_sql(connection, query, values)

def view_data(conn: ConnectionManager, table_name: str, columns: Optional[List[str]] = None, 
              condition: Optional[str] = None, params: Optional[tuple] = None) -> List[Dict]:
    """
    Flexible data retrieval with optional filtering
    
    :param conn: Active connection manager
    :param table_name: Table to query
    :param columns: List of columns to retrieve (None for all)
    :param condition: WHERE clause (without WHERE keyword)
    :param params: Parameters for condition clause
    :return: List of dictionaries with column names as keys
    """
    cols = sql.SQL('*') if not columns else \
        sql.SQL(', ').join(map(sql.Identifier, columns))
        
    query = sql.SQL("SELECT {} FROM {}").format(
        cols,
        sql.Identifier(table_name)
    )
    
    if condition:
        query = sql.SQL("{} WHERE {}").format(query, sql.SQL(condition))
    
    with conn as connection:
        return execute_sql(connection, query, params, fetch=True)

def update_data(conn: ConnectionManager, table_name: str, data: Dict[str, Any], 
                condition: str, condition_params: Optional[tuple] = None) -> None:
    """
    Update records with conditional safety
    
    :param conn: Active connection manager
    :param table_name: Table to update
    :param data: Dict of {column: new_value}
    :param condition: WHERE clause
    :param condition_params: Parameters for condition
    """
    if not data:
        raise ValueError("Update data cannot be empty")
        
    set_clause = sql.SQL(', ').join(
        sql.SQL("{} = {}").format(
            sql.Identifier(k),
            sql.Placeholder()
        ) for k in data
    )
    
    values = list(data.values())
    if condition_params:
        values.extend(condition_params)
    
    query = sql.SQL("UPDATE {} SET {} WHERE {}").format(
        sql.Identifier(table_name),
        set_clause,
        sql.SQL(condition)
    )

    with conn as connection:
        execute_sql(connection, query, values)

def delete_data(conn: ConnectionManager, table_name: str, condition: str, 
                params: Optional[tuple] = None) -> None:
    """
    Safe deletion with mandatory conditions
    
    :param conn: Active connection manager
    :param table_name: Table to delete from
    :param condition: WHERE clause (required to prevent mass deletion)
    :param params: Parameters for condition
    """
    if not condition:
        raise ValueError("Condition is required for deletion")
        
    query = sql.SQL("DELETE FROM {} WHERE {}").format(
        sql.Identifier(table_name),
        sql.SQL(condition)
    )

    with conn as connection:
        execute_sql(connection, query, params)

def drop_table(conn: ConnectionManager, table_name: str) -> None:
    """Drop table(s) with existence check (accepts comma-separated table names)"""
    tables = [t.strip() for t in table_name.split(',')]
    with conn as connection:
        for table in tables:
            query = sql.SQL("DROP TABLE IF EXISTS {}").format(sql.Identifier(table))
            execute_sql(connection, query)

def show_tables(conn: ConnectionManager) -> List[str]:
    """List all tables in current database"""
    query = """
        SELECT table_name 
        FROM information_schema.tables 
        WHERE table_schema = 'public'
    """
    
    with conn as connection:
        results = execute_sql(connection, sql.SQL(query), fetch=True)
        return [row['table_name'] for row in results] if results else []

def show_columns(conn: ConnectionManager, table_name: str) -> List[Dict]:
    """Get column names and data types for a table"""
    query = """
        SELECT column_name, data_type
        FROM information_schema.columns
        WHERE table_name = %s
    """
    
    with conn as connection:
        return execute_sql(connection, sql.SQL(query), (table_name,), fetch=True)

def query(conn: ConnectionManager, operation: Union[str, Any], **kwargs) -> Any:
    """
    Unified query interface with built-in operations and raw SQL support
    
    Supported operations:
    - 'create': (table_name, columns)
    - 'insert': (table_name, data)
    - 'select': (table_name, [columns], [condition], [params])
    - 'update': (table_name, data, condition, [condition_params])
    - 'delete': (table_name, condition, [params])
    - 'drop': (table_name)
    - 'tables': ()
    - 'columns': (table_name)
    - 'create_db': (db_name)
    
    For raw SQL:
    - query(conn, "SELECT * FROM table WHERE id = %s", params=(1,))
    
    :param conn: Active connection manager
    :param operation: Operation name string or raw SQL query
    :param kwargs: Operation-specific parameters or query params
    :return: Results for SELECT queries, None otherwise
    """
    operations = {
        'create': create_table,
        'insert': insert_data,
        'select': lambda c, **k: view_data(c, k['table_name'], k.get('columns'), k.get('condition'), k.get('params')),
        'update': update_data,
        'delete': delete_data,
        'drop': drop_table,
        'tables': lambda c, **_: show_tables(c),
        'columns': lambda c, **k: show_columns(c, k['table_name']),
        'create_db': lambda c, **k: create_db(c, k['db_name'])
    }
    
    # Handle raw SQL queries
    if isinstance(operation, str) and operation.strip().upper().split()[0] in (
        'SELECT', 'INSERT', 'UPDATE', 'DELETE', 'CREATE', 'DROP'):
        
        sql_command = operation.strip().upper().split()[0]
        fetch = sql_command == 'SELECT'
        params = kwargs.get('params')
        
        with conn as connection:
            return execute_sql(connection, sql.SQL(operation), params, fetch)
    
    # Handle named operations
    if operation not in operations:
        raise ValueError(f"Invalid operation. Choose from: {list(operations.keys())}")
    
    return operations[operation](conn, **kwargs)